(function () {
  'use strict';

  angular.module('inventoryMonitoringApp', ['ngRoute'])
         .config(['$routeProvider', '$compileProvider', '$locationProvider', 'VEEVA_MESSAGES', AppConfig])
         .constant("moment", moment);

  function AppConfig($routeProvider, $compileProvider, $locationProvider, VEEVA_MESSAGES){
    $locationProvider.hashPrefix('');
    // begin special case for windows platform
    $compileProvider.aHrefSanitizationWhitelist(/^\s*(https?|ftp|mailto|file|ghttps?|ms-appx|x-wmapp0|ms-local-stream):/);
    $compileProvider.imgSrcSanitizationWhitelist(/^\s*(https?|ms-local-stream|file|data):/);
    // end special case for windows platform


    var requiredMessages = VEEVA_MESSAGES;
    var requiredMessagesList = [];
    var translatedMessages = {};
    var ds = window.ds;

    angular.forEach(requiredMessages, function(msg) {
      requiredMessagesList.push(msg);
    });

    var originalWhen = $routeProvider.when;

    $routeProvider.when = function(path, route) {
      route.resolve = {
        globalData: ['$q', 'AccountService', 'UserService', 'InvMonitoringService', 'MessageService', 'Utility', function($q, accountService, userService, invMonitoringService, messageService, utility) {
          var deferred = $q.defer();
          var keyword = 'InventoryMonitoring';

          userService.fetchCurrentUser().then(function() {

            ds.getDataForCurrentObject('Account', 'Id').then(function(acctResponse) {

              accountService.setCurrentAccountId(acctResponse.Account.Id);

              ds.getDataForCurrentObject(keyword, 'Id').then(function (response) {
                if (response.success) {

                  invMonitoringService.setCurrentIMId(response.InventoryMonitoring.Id);

                  invMonitoringService.fetchInventoryMonitoring().then(function () {

                    if (messageService.getTranslatedMessages()) {
                      deferred.resolve();
                    } else {
                      var defaultLanguageLocale = 'en_US';
                      var languageLocale = userService.getCurrentUser().LanguageLocaleKey || defaultLanguageLocale;

                      messageService.fetchTranslations(requiredMessagesList, languageLocale).then(
                        function (rawTranslations) {
                          if (!rawTranslations.length) {
                            messageService.fetchTranslations(requiredMessagesList, defaultLanguageLocale)
                                          .then(function (rawTranslations) {
                                            setUpTranslations(rawTranslations);
                                            deferred.resolve();
                                          });
                          } else {
                            setUpTranslations(rawTranslations);
                            deferred.resolve();
                          }
                        }, utility.genericQueryErrorHandler(deferred));
                      }
                  });
                } else {
                  (utility.genericQueryErrorHandler(deferred))(
                    response.message);
                }
              }, utility.genericQueryErrorHandler(deferred));

            }, utility.genericQueryErrorHandler(deferred));
          }, function(e){
            console.log('user failed' + e);
          });

          function setUpTranslations(rawTranslations) {
            angular.forEach(requiredMessages, function(msg, key) {
              var foundTranslation = _.find(rawTranslations, function(rawMsg) {
                return msg.msgName === rawMsg.Name.value && msg.msgCategory === rawMsg.Category_vod__c.value;
              });

              if(foundTranslation) {
                translatedMessages[key] = foundTranslation.Text_vod__c.display;
              }
            });
            messageService.setTranslatedMessages(translatedMessages);
          }

          return deferred.promise;
        }]
      };
      return originalWhen.call($routeProvider, path, route);
    };

    $routeProvider
      .when('/contracts', {
        templateUrl: 'views/contracts.html',
        controller: 'ContractsController',
        controllerAs: 'contract'
      });
    }
}());


