(function () {
  'use strict';

  angular.module('inventoryMonitoringApp')
         .controller('ContractsController', [
           '$scope', '$timeout', '$q', 'AccountService', 'InvMonitoringService', 'InvMonitoringLineSvc', 'ProductService',
           'BrandService', 'OrderService', 'OrderLineService', 'ContractService', 'ContractLineService', 'MessageService', 'Utility', ContractCtrl]);

  function ContractCtrl($scope, $timeout, $q, accountService, invMonitoringService, invMonitoringLineSvc, productService,
                        brandService, orderService, orderLineService, contractService, contractLineService, messageService, utility) {
    var contract = this;

    contract.errorOccurs = false;

    // Data models
    contract.brands = [];
    contract.products = [];

    contract.startColor = '#5AC8FA';
    contract.endColor = '#008FD2';

    contract.getDoughnutStyle = function() {
      return {
        width: '140px',
        height: '140px',
        display: 'inline-block'
      };
    };

    var currentAccountId = accountService.getCurrentAccountId();
    var currentIMId = invMonitoringService.getCurrentIMId();
    var currentContractId = invMonitoringService.getInventoryMonitoringField('Contract_vod__c');
    var IMDate = invMonitoringService.getInventoryMonitoringField('Inventory_Monitoring_Date_vod__c');
    var invProductIds;
    var isFutureOrders = true;

    // Promises
    var brandDataDeferred = $q.defer();
    var productDataDeferred = $q.defer();
    var invMonitoringLinesDeferred = $q.defer();
    var contractLineDeferred = $q.defer();
    var orderLineDeferred = $q.defer();
    var contractDeferred = $q.defer();
    var orderDeferred = $q.defer();

    var allPromises = [
      brandDataDeferred.promise,
      productDataDeferred.promise,
      invMonitoringLinesDeferred.promise,
      contractLineDeferred.promise,
      orderLineDeferred.promise,
      contractDeferred.promise,
      orderDeferred.promise
    ];

    $scope.$emit('reportLanded');

    if (!currentContractId) {
      contract.errorOccurs = true;
      return;
    }

    invMonitoringLineSvc.fetchInventoryMonitoringLines(currentIMId).then(function(lines) {
      invProductIds = invMonitoringLineSvc.getProductIdsFromInvLines();

      contract.lines = lines;
      contract.tableLabels = invMonitoringLineSvc.getTableLabels();

      productService.fetchProducts(invProductIds).then(function(products){
        var brandsId = productService.getAllParentIds();

        contract.products = products;

        brandService.fetchBrands(brandsId).then(function(brands){
          contract.brands = brands;

          brandDataDeferred.resolve();
        }, utility.genericQueryErrorHandler(brandDataDeferred));

        productDataDeferred.resolve();
      }, utility.genericQueryErrorHandler(productDataDeferred));

      invMonitoringLinesDeferred.resolve();
    }, utility.genericQueryErrorHandler(invMonitoringLinesDeferred));

    invMonitoringLinesDeferred.promise.then(function(){
      orderService.fetchOrders(currentAccountId, isFutureOrders).then(function(futureOrders) {

        if (futureOrders.length === 0) {
          fetchOrderNotInFuture(currentAccountId, !isFutureOrders, invProductIds, orderDeferred, orderLineDeferred);
        } else {
          contract.orders = futureOrders;

          orderLineService.fetchOrderLines(invProductIds, orderService.getAllOrderIds()).then(function(orderLines) {

            if (orderLines.length === 0) {
              fetchOrderNotInFuture(currentAccountId, !isFutureOrders, invProductIds, orderDeferred, orderLineDeferred);
            } else {
              contract.orderLines = orderLineService.mergeOrderDateFromOrder(contract.orders);

              orderLineDeferred.resolve();
              orderDeferred.resolve();
            }
          }, utility.genericQueryErrorHandler(orderLineDeferred));
        }
      }, utility.genericQueryErrorHandler(orderDeferred));
    });

    contractService.fetchContract(currentContractId).then(function(contractResponse) {
      contract.contractData = contractResponse;
      contract.contractName = contractService.getName();

      contractDeferred.resolve();
    });

    contractLineService.fetchContractLines(currentContractId, IMDate).then(function(contractLines) {
      contract.contractLines = contractLines;

      contractLineDeferred.resolve();
    });

    $q.all(allPromises).then(function() {
      console.log(contract);

      renderSections();
      contract.allReady = true;
    });

    contract.brandDataUpdate = function(brands) {
      contract.products = productService.getProductsForFilter(brands);

      renderSections();
      $timeout(function() {
        $scope.$broadcast('updateDoughnutChart');
      });
    };

    contract.productDataUpdate = function(selectedBrand, selectedProduct) {

      if (selectedProduct.Id) {
        contract.products = productService.getSpecificProduct(selectedProduct);
      }else {
        // all products
        contract.products = productService.getProductsForFilter(selectedBrand);
      }

      renderSections();

      $timeout(function() {
        $scope.$broadcast('updateDoughnutChart');
      });
    };

    function initiateNonCompliantTableData() {
      contract.nonCompliantTable = {
        headers: utility.getCompliantTableHeaders(
          contract.tableLabels,
          messageService.getMessage('orderData')),
        data: contract.compliantGroup.nonCompliant,
        filter: {}
      };
    }

    function initiateCompliantTableData() {
      contract.compliantTable = {
        headers: utility.getCompliantTableHeaders(
          contract.tableLabels,
          messageService.getMessage('orderData')),
        data: contract.compliantGroup.compliant,
        filter: {}
      };
    }

    function initiateNonContractedTableData() {
      contract.nonContractedTable = {
        headers: utility.getCompliantTableHeaders(
          contract.tableLabels,
          messageService.getMessage('orderData')),
        data: contract.compliantGroup.nonContracted,
        filter: {}
      };
    }

    function renderSections() {
      var mergedProduct = utility.mergeDataToProducts(contract.products, contract.lines, contract.contractLines, contract.orderLines, orderService.getIsFutureOrderFlag());

      contract.compliantGroup = utility.groupCompliantProducts(mergedProduct);

      contract.contractedInvLines = contract.compliantGroup.compliant.concat(contract.compliantGroup.nonCompliant);

      contract.currentSelectedInvLines = invMonitoringLineSvc.getLinesFromProductIds(productService.extractIds(contract.products));

      contract.currentSelectedContractLines = utility.getCurrentSelectedContractLines(mergedProduct);

      initiateNonCompliantTableData();
      initiateCompliantTableData();
      initiateNonContractedTableData();
    }

    function fetchOrderNotInFuture(acctId, isFutureOrdersFlag, invProductIds, orderDeferObj, orderLineDeferredObj) {
      orderService.fetchOrders(acctId, isFutureOrdersFlag).then(function(pastOrders) {
        var orderIds = orderService.getAllOrderIds();
        contract.orders = pastOrders;

        orderLineService.fetchOrderLines(invProductIds, orderIds).then(function() {
          contract.orderLines = orderLineService.mergeOrderDateFromOrder(contract.orders);
          orderLineDeferredObj.resolve();
        }, utility.genericQueryErrorHandler(orderLineDeferred));

        orderDeferObj.resolve();
      });
    }
  }
}());
