(function() {
  'use strict';

  angular.module('inventoryMonitoringApp').directive('contractFilter', directiveFn);

  function directiveFn() {
    var controller = ['$scope', 'BrandService', 'ProductService', 'Utility', 'FilterType', 'MessageService',
      function($scope, brandService, productService, utility, filterType, messageService) {
        var filter = this;

        function initialize() {
          filter.brands = [];
          filter.products = [];
          filter.selectedBrand = null;
          filter.selectedProduct = null;
          filter.filterRecords = {};

          getBrands();
          getProducts();
        }

        /**
         *  Local functions
         */

        filter.brandUpdate = function() {
          if(filter.selectedBrand) {
            if(filter.selectedBrand.Id) {
              // one brand for parent
              $scope.brandDataUpdate({brands: filter.selectedBrand});
            } else {
              // all brands for parent
              $scope.brandDataUpdate({brands: brandService.getAllBrands()});
            }
            getProducts();
          }
        };

        filter.productUpdate = function() {
          if (filter.selectedProduct) {
            if(!filter.selectedProduct.Id) {
              // all products
              getProducts();
            }

            $scope.productDataUpdate({selectedBrand: filter.selectedBrand, selectedProduct: filter.selectedProduct});
          }
        };

        filter.reset = function() {
          getBrands();
          getProducts();

          $scope.productDataUpdate({selectedBrand: filter.selectedBrand, selectedProduct: filter.selectedProduct});
        };

        /**
         *  Private functions
         */
        function getBrands() {
          var brands = brandService.getAllBrands();
          var sortedBrands = _.sortBy(brands, ['Name']);
          var allBrandsMessage = messageService.getMessage('allBrands');
          var allBrandsLabel = {
            Name: allBrandsMessage
          };

          sortedBrands.unshift(allBrandsLabel);

          filter.brands = sortedBrands;
          filter.selectedBrand = filter.brands[0];
        }

        function getProducts() {
          var prods = productService.getProductsForFilter(filter.selectedBrand);
          var sortedProducts = _.sortBy(prods, ['Name']);
          var allProductsMessage = messageService.getMessage('allProducts');
          var allProductsLabel = {
            Name: allProductsMessage
          };

          sortedProducts.unshift(allProductsLabel);

          filter.products = sortedProducts;
          filter.selectedProduct = filter.products[0];
        }

        initialize();
      }];

    return {
      templateUrl: 'views/contract-filter.html',
      restrict: 'E',
      scope: {
        productDataUpdate: '&',
        brandDataUpdate: '&'
      },
      controller: controller,
      controllerAs: 'filter'
    };
  }
})();
