(function () {
  'use strict';

  angular.module('inventoryMonitoringApp').directive('doughnutChart', function() {
    return {
      template: '<canvas></canvas>',
      restrict: 'E',
      replace: true,
      scope: {
        color1: '@',
        color2: '@',
        data1: '=',
        data2: '=',
        secondRowText: '@'
      },
      link: function(scope, element) {
        var gradient = element[0].getContext("2d").createLinearGradient(0, 0, 0, 400);

        gradient.addColorStop(0, scope.color1);
        gradient.addColorStop(1, scope.color2);

        var data = {
          datasets: [
            {
              data: [formatData().data1, formatData().data2],
              backgroundColor: [gradient, "#e0e0e0"]
            }
          ]
        };

        var options = {
          cutoutPercentage: 80,
          responsive: false,
          events: [],
          customLabel: calculatePercentage() + '%',
          customLabelColor: '#666666',
          secondRowText: scope.secondRowText,
          secondRowColor: '#999999',
          centerLabel: true
        };

        if (scope.semiCircle) {
          options.semiCircle = true;
          options.rotation = 1 * Math.PI;
          options.circumference = 1 * Math.PI;
        }

        // Format data to make sure the doughnut chart looks correct
        function formatData() {
          var data1 = +scope.data1 || 0;
          var data2 = +scope.data2 || 0;

          // Avoid case that the doughnut will be gone if the second data is 0. i.e. 0/0
          if(data1 === 0 && data2 === 0) {
            data2 = 1;
          } else {
            data1 = data1/data2;
            data2 = (data2-scope.data1)/data2;
          }

          return {
            data1: data1,
            data2: data2
          };
        }

        function calculatePercentage() {
          var num1 = +scope.data1;
          var num2 = +scope.data2;

          if(num2 === 0) {
            return 0;
          } else {
            return Math.floor(100* num1/num2);
          }
        }

        scope.$on('updateDoughnutChart', function() {
          console.log('Doughnut data: ' + scope.data1 + ',' + scope.data2);

          doughnutChart.data.datasets[0].data = [formatData().data1, formatData().data2];
          doughnutChart.options.customLabel = calculatePercentage() + '%';
          doughnutChart.update();
        });

        var doughnutChart = new Chart(element, {
          type: 'doughnut',
          data: data,
          options: options
        });
      }
    };
  });
}());
