(function() {
  'use strict';

  angular.module('inventoryMonitoringApp').directive('reportTable', directiveFn);

  function directiveFn() {
    var linker = function(scope, element) {
      if(!scope.width) {
        scope.width = '100%';
      }
      element.css({width: scope.width});
    };

    var controller = ['$scope', 'Utility', function($scope, utility) {
      var table = this;

      table.sortColumn = function(columnName) {
        if($scope.columnSortable) {
          table.columnSorted = columnName;

          if(table.orderKey === table.columnSorted) {
            table.orderKey = '-' + table.columnSorted;
          } else {
            table.orderKey = table.columnSorted;
          }

          console.log('Sorting column based on key: ' + table.orderKey);
        }
      };

      table.filterFunction = function(row) {
        if(angular.isFunction($scope.customFilterFunction())) {
          return $scope.customFilterFunction()(row);
        } else {
          return true;
        }
      };

      table.applySortingIcon = function() {
        if(angular.isString(table.orderKey) && table.orderKey[0] === '-') {
          return 'fa-caret-down';
        } else {
          return 'fa-caret-up';
        }
      };

      table.isCompliant = function (actualValue, contractedValue, isFacing) {
        if (isFacing) {
          return actualValue >= contractedValue;
        }

        if (!contractedValue && contractedValue !== 0) {
          return true;
        }

        return actualValue === contractedValue;
      };

      table.noHighlightColor = function(actualValue, contractedValue) {
        return utility.isNull(actualValue) && utility.isNull(contractedValue);
      };
    }];

    return {
      templateUrl: 'views/report-table.html',
      restrict: 'E',
      replace: true,
      scope: {
        headers: '=',
        data: '=',
        width: '@',
        clickable: '=',
        columnSortable: '=',
        customFilterFunction: '&',
        rowClickHandler: '&',
        contracted: '='
      },
      link: linker,
      controller: controller,
      controllerAs: 'table'
    };
  }
})();
