(function() {
  'use strict';

  angular.module('inventoryMonitoringApp').factory('ChartJsPlugins', [factoryFn]);

  function factoryFn() {
    var plugins = this;

    function isPluginEnabled(chartInstance, pluginName) {
      return chartInstance.options[pluginName] && (typeof chartInstance.options[pluginName] === 'object' || typeof chartInstance.options[pluginName] === 'boolean');
    }

    function drawStrip(ctx, x1, y1, x2, y2, strokeWidth, color) {
      ctx.beginPath();
      ctx.moveTo(x1, y1);
      ctx.lineTo(x2, y2);
      ctx.lineWidth = strokeWidth;
      ctx.strokeStyle = color;
      ctx.stroke();
    }

    function drawText(ctx, x, y, strokeWidth, text, color, fontSize) {
      ctx.font = fontSize + " 'Lato'";
      ctx.textAlign = "center";
      ctx.lineWidth = strokeWidth;
      ctx.fillStyle = color;
      ctx.fillText(text, x, y);
    }

    plugins.backgroundStrip = {
      pluginName: 'backgroundStrip',
      defaultColor: '#fafafb',
      beforeDraw: function(chartInstance) {
        if(!isPluginEnabled(chartInstance, this.pluginName)) {
          return;
        }
        var canvas = chartInstance.chart.ctx.canvas;
        var ctx = canvas.getContext('2d');
        var constraints = {
          'left': chartInstance.chartArea.left,
          'right': chartInstance.chartArea.right
        };
        var scale = chartInstance.scales['y-axis-0'];
        var strokeWidth = scale.height/(scale.ticks.length - 1);
        var yPos = scale.top + scale.paddingTop + strokeWidth/2;
        var color = this.defaultColor;
        var stripOptions = chartInstance.options[this.pluginName];
        if(typeof stripOptions.color === 'string') {
          color = stripOptions.color;
        }
        var odd = false;
        while(yPos < scale.height) {
          if(odd) {
            drawStrip(ctx, constraints.left, yPos, constraints.right, yPos, strokeWidth, color);
          }
          odd = !odd;
          yPos = yPos + strokeWidth;
        }
      }
    };

    plugins.backgroundText = {
      pluginName: 'backgroundText',
      defaultColor: 'rgba(153,153,153,0.6)',
      defaultFontSize: '16px',

      afterDraw: function(chartInstance) {
        if(!isPluginEnabled(chartInstance, this.pluginName)) {
          return;
        }

        var canvas = chartInstance.chart.ctx.canvas;
        var ctx = canvas.getContext('2d');
        var constraints = {
          'top': chartInstance.chartArea.top,
          'left': chartInstance.chartArea.left,
          'bottom': chartInstance.chartArea.bottom,
          'right': chartInstance.chartArea.right
        };

        var color = this.defaultColor;
        var fontSize = this.defaultFontSize;
        var text = '';

        var options = chartInstance.options[this.pluginName];
        if(typeof options.color === 'string') {
          color = options.color;
        }
        if(typeof options.fontSize === 'string') {
          fontSize = options.fontSize;
        }
        if(typeof options.text === 'string') {
          text = options.text;
        }

        var textX = (constraints.left + constraints.right) / 2;
        var textY = (constraints.top + constraints.bottom) / 2;
        drawText(ctx, textX, textY, 1, text, color, fontSize);
      }
    };

    plugins.centerLabel = {
      pluginName: 'centerLabel',

      beforeDraw: function(chart) {
        if(!isPluginEnabled(chart, this.pluginName)) {
          return;
        }

        if(chart.config.type === 'doughnut') {
          var width = chart.chart.width;
          var height = chart.chart.height;
          var ctx = chart.chart.ctx;

          ctx.restore();

          var fontSize = (height / 70).toFixed(2);
          ctx.font = "normal normal 100 "+ fontSize + "em 'Lato-Light'";
          ctx.textBaseline = "middle";

          if(chart.config.options.customLabelColor) {
            ctx.fillStyle = chart.config.options.customLabelColor;
          }

          var text = chart.config.options.customLabel;
          var textX = Math.round((width - ctx.measureText(text).width) / 2);
          var textY = chart.config.options.semiCircle ? height / 4 * 3 - 18 : height / 2 - 10;

          ctx.fillText(text, textX, textY);

          var smallerFont = (height / 140).toFixed(2);
          ctx.font = "normal normal 100 "+ smallerFont + "em 'Lato'";
          ctx.textBaseline = "middle";

          if(chart.config.options.secondRowColor) {
            ctx.fillStyle = chart.config.options.secondRowColor;
          }

          var secondText = chart.config.options.secondRowText;
          var xPoint = Math.round((width - ctx.measureText(secondText).width) / 2);
          var yPoint = chart.config.options.semiCircle ? height / 4 * 3 + 36 : height / 2 + 48;

          ctx.fillText(secondText, xPoint, yPoint);

          ctx.save();
        }
      }
    };

    return plugins;
  }
})();
