(function() {
  'use strict';

  angular.module('inventoryMonitoringApp').factory('ContractLineService', ['$q', 'Endpoints', 'Utility', 'ContractService', factoryFn]);

  function factoryFn($q, endpoints, utility, contractService) {
    var service = this;
    var contractLines = {};
    var inventoryMonitoringDate;

    function findValidLines(line) {
      var startDateToUse, endDateToUse;

      if (utility.isNull(line.Start_Date_vod__c.value) && utility.isNull(line.End_Date_vod__c.value)) {
        return line;
      }

      startDateToUse = line.Start_Date_vod__c.value || contractService.getStartDate();
      endDateToUse = line.End_Date_vod__c.value || contractService.getEndDate();

      if (moment(startDateToUse).isSameOrBefore(inventoryMonitoringDate) && moment(endDateToUse).isSameOrAfter(inventoryMonitoringDate)) {
        return line;
      }
    }

    service.fetchContractLines = function(contractId, IMDate) {
      var deferred = $q.defer();
      inventoryMonitoringDate = IMDate;

      if(contractLines.data) {
        deferred.resolve(_.cloneDeep(contractLines.data));
      } else {
        endpoints.fetchContractLines(contractId).then(function(response) {
          var contractLineData;

          contractLines = response;
          contractLineData = utility.normalizeArray(contractLines, 'data');
          contractLineData = contractLineData.filter(findValidLines);

          deferred.resolve(contractLineData);
        }, utility.genericQueryErrorHandler(deferred));
      }

      return deferred.promise;
    };

    return service;
  }
})();
