(function () {
  'use strict';

  angular.module('inventoryMonitoringApp').factory('Endpoints', ['$q', 'queries', 'ObjectFieldSecurity', 'Utility', function ($q, queries, objectFieldSecurity, utility) {
    var endpoints = this;
    var ds = window.ds;
    var utils = new window.VeevaUtilities();

    endpoints.fetchCurrentUser = function() {
      var deferred = $q.defer();

      ds.getDataForCurrentObject('User', 'Id').then(function(resp1) {
        if(resp1.success) {
          var currentUser = resp1.User;
          var queryConfig = angular.copy(queries.user);
          queryConfig.where = queryConfig.where.replace(/_userId/, '\'' + currentUser.Id + '\'');

          ds.queryRecord(queryConfig).then(function (resp2) {
            deferred.resolve(resp2);
          }, utility.genericQueryErrorHandler(deferred));

        } else {
          (utility.genericQueryErrorHandler(deferred))(resp1.message);
        }
      });

      return deferred.promise;
    };

    endpoints.fetchInventoryMonitoring = function(id) {
      var deferred = $q.defer();
      var queryConfig = angular.copy(queries.inventoryMonitoring);

      queryConfig.where = queryConfig.where.replace(/_id/, '\'' + id + '\'');

      objectFieldSecurity.findAvailableFields(queries.inventoryMonitoring).then(function(fields){
        queryConfig.fields = fields;

        ds.queryRecord(queryConfig).then(function(resp) {
          deferred.resolve(resp);
        }, utility.genericQueryErrorHandler(deferred));
      });

      return deferred.promise;
    };

    endpoints.fetchInventoryMonitoringLines = function(inventoryMonitoringId) {
      var deferred = $q.defer();
      var queryConfig = angular.copy(queries.inventoryMonitoringLine);

      queryConfig.where = queryConfig.where.replace(/_invMonitoringId/, '\'' + inventoryMonitoringId + '\'');

      objectFieldSecurity.findAvailableFields(queries.inventoryMonitoringLine).then(function(fields){
        queryConfig.fields = fields;

        ds.runQuery(queryConfig).then(function(resp) {
          deferred.resolve(resp);
        }, utility.genericQueryErrorHandler(deferred));
      });

      return deferred.promise;
    };

    endpoints.fetchAllBrands = function(brandIds) {
      var deferred = $q.defer();
      var queryConfig = angular.copy(queries.allBrands);

      objectFieldSecurity.findAvailableFields(queries.allBrands).then(function(fields){
        queryConfig.fields = fields;
        queryConfig.where += ds.getInStatement(brandIds);

        ds.queryRecord(queryConfig).then(function(resp) {
          deferred.resolve(resp);
        }, utility.genericQueryErrorHandler(deferred));
      });

      return deferred.promise;
    };

    endpoints.fetchProducts = function(productIds) {
      var deferred = $q.defer();
      var queryConfig = angular.copy(queries.products);

      objectFieldSecurity.findAvailableFields(queries.products).then(function(fields){
        queryConfig.fields = fields;
        queryConfig.where += ds.getInStatement(productIds);

        ds.queryRecord(queryConfig).then(function(resp) {
          deferred.resolve(resp);
        }, utility.genericQueryErrorHandler(deferred));
      });

      return deferred.promise;
    };

    endpoints.fetchContract = function(contractId) {
      var deferred = $q.defer();
      var queryConfig = angular.copy(queries.contract);
      var metaQueryConfig = _.cloneDeep(queries.contract);
      delete metaQueryConfig.where;

      queryConfig.where = queryConfig.where.replace(/_id/, '\'' + contractId + '\'');

      objectFieldSecurity.findAvailableFields(metaQueryConfig).then(function(fields){
        queryConfig.fields = fields;

        ds.queryRecord(queryConfig).then(function(resp) {
          deferred.resolve(resp);
        }, utility.genericQueryErrorHandler(deferred));
      }, utility.genericQueryErrorHandler(deferred));

      return deferred.promise;
    };

    endpoints.fetchContractLines = function(contractId) {
      var deferred = $q.defer();
      var queryConfig = angular.copy(queries.contractLine);
      var metaQueryConfig = _.cloneDeep(queries.contractLine);
      delete metaQueryConfig.where;

      queryConfig.where = utils.replaceStringInWhereClause(queryConfig.where, contractId, '_contractId');

      objectFieldSecurity.findAvailableFields(metaQueryConfig).then(function(fields){
        queryConfig.fields = fields;

        ds.runQuery(queryConfig).then(function(resp) {
          deferred.resolve(resp);
        }, utility.genericQueryErrorHandler(deferred));
      }, utility.genericQueryErrorHandler(deferred));

      return deferred.promise;
    };

    endpoints.fetchOrders = function(accountId, isFutureOrders, todayDate) {
      var deferred = $q.defer();
      var queryConfig = angular.copy(queries.order);
      var whereClauseEmptyValue = utils.isOnline() ? null : "''";
      var operand = isFutureOrders ? '>' : '<=';
      todayDate = utils.isOnline() ? todayDate : '\'' + todayDate + '\'';

      objectFieldSecurity.findAvailableFields(queries.order).then(function(fields){
        queryConfig.fields = fields;

        queryConfig.where = queryConfig.where.replace(/_accountId/, '\'' + accountId + '\'');
        queryConfig.where = queryConfig.where.replace(/_empty/, whereClauseEmptyValue);
        queryConfig.where = queryConfig.where.replace(/_operand/, operand );
        queryConfig.where = queryConfig.where.replace(/_today/, todayDate);

        ds.queryRecord(queryConfig).then(function(resp) {
          deferred.resolve(resp);
        }, utility.genericQueryErrorHandler(deferred));
      });

      return deferred.promise;
    };

    endpoints.fetchOrderLines = function(productIds, orderIds) {
      var deferred = $q.defer();
      var queryConfig = angular.copy(queries.orderLine);
      var productIdForInClasue = ds.getInStatement(productIds);
      var tempQuery;

      objectFieldSecurity.findAvailableFields(queries.orderLine).then(function(fields){
        queryConfig.fields = fields;
        queryConfig.where = queryConfig.where.replace(/_productIds/, productIdForInClasue);
        queryConfig.where += ds.getInStatement(orderIds);

        // short term fix multiple IN statement parsing problem in WM
        // running two times because replace only replace first occurrence for string
        // this could be removed after long term fixed implemented
        if (!utils.isOnline() && utils.isWin8()) {
          tempQuery = queryConfig.where.replace('{', '(');
          tempQuery = tempQuery.replace('{', '(');
          tempQuery = tempQuery.replace('}', ')');
          queryConfig.where = tempQuery.replace('}', ')');
        }

        ds.queryRecord(queryConfig).then(function(resp) {
          deferred.resolve(resp);
        }, utility.genericQueryErrorHandler(deferred));
      });

      return deferred.promise;
    };

    endpoints.getTranslations = function(messagesToGet, userLocale) {
      var deferred = $q.defer();

      ds.getVeevaMessagesWithDefault(messagesToGet, userLocale).then(function (resp) {
        deferred.resolve(resp);
      }, utility.genericQueryErrorHandler(deferred));

      return deferred.promise;
    };

    endpoints.fetchCurrencyIsoCode = function(messagesToGet, userLocale) {
      var deferred = $q.defer();

      ds.getVeevaMessagesWithDefault(messagesToGet, userLocale).then(function (resp) {
        deferred.resolve(resp);
      }, utility.genericQueryErrorHandler(deferred));

      return deferred.promise;
    };

    return endpoints;
  }]);

}());
