(function() {
  'use strict';

  angular.module('inventoryMonitoringApp').factory('InvMonitoringLineSvc', ['$q', 'Endpoints', 'Utility', 'UserService', 'AppConstant', factoryFn]);

  function factoryFn($q, endpoints, utility, userService, appConstant) {
    var service = this;
    var inventoryLines = {};
    var veevaUtils = new window.VeevaUtilities();
    var tableLabels = [
      'Product_vod__c',
      'Facings_simple_vod__c',
      'Position_vod__c',
      'Layer_vod__c',
      'Consumer_Price_vod__c',
      'Inventory_vod__c'
    ];

    var currentUser = userService.getCurrentUser();

    function getFormattedNumber(locale, priceValue) {
      if (!priceValue && priceValue !== 0) {
        return priceValue;
      }

      return new Intl.NumberFormat(locale, appConstant.DECIMAL_NUMBER_CONFIG).format(priceValue);
    }

    function trimLocale(locale) {
      var codeList = locale.split('-');

      if (codeList.length > 1) {
        return codeList.slice(0, 2).join('-');
      }

      return locale;
    }

    function formatNumber(lines, locale) {
      return lines.map(function(line){
        var priceObj = line.Consumer_Price_vod__c;

        try {
          priceObj.display = getFormattedNumber(locale, priceObj.value);
        }
        catch (e){
          try {
            priceObj.display = getFormattedNumber(trimLocale(locale), priceObj.value);
          }
          catch(error) {
            console.error(appConstant.NUMBER_FORMAT_LOCALE_ERROR + error.message);
            priceObj.display = getFormattedNumber(appConstant.DEFAULT_LOCALE, priceObj.value);
          }
        }

        return line;
      });
    }

    function standardizeLocaleCode(localeCode) {
      // because Intl.NumberFormat cannot recognize en_US
      // replacing _ with -, eg: en_US to en-US
      return localeCode ? localeCode.replace(new RegExp('_','g'),'-') : appConstant.DEFAULT_LOCALE;
    }

    service.fetchInventoryMonitoringLines = function(inventoryMonitoringId) {
      var deferred = $q.defer();

      if(inventoryLines.data) {
        deferred.resolve(_.cloneDeep(inventoryLines.data));
      } else {
        endpoints.fetchInventoryMonitoringLines(inventoryMonitoringId).then(function(response) {
          var invLines;

          inventoryLines = response;
          invLines = utility.normalizeArray(inventoryLines, 'data');

          // short term fix until WM change platform behavior
          if (!veevaUtils.isWin8() && !veevaUtils.isOnline()) {
            invLines = formatNumber(invLines, standardizeLocaleCode(currentUser.LocaleSidKey));
          }

          deferred.resolve(invLines);

        }, utility.genericQueryErrorHandler(deferred));
      }

      return deferred.promise;
    };

    service.getProductIdsFromInvLines = function() {
      var invLines = utility.normalizeArray(inventoryLines, 'data');

      return utility.extractProductId(invLines);
    };

    service.getTableLabels = function() {
      return inventoryLines.fieldLabels.filter(function(field){
        return tableLabels.indexOf(field.name) !== -1;
      });
    };

    service.getLinesFromProductIds = function(productIds) {
      var invLines = utility.normalizeArray(inventoryLines, 'data');

      return invLines.filter(function(line){
        return productIds.indexOf(line.Product_vod__c.value) !== -1;
      });
    };

    return service;
  }
})();
