(function() {
  'use strict';

  angular.module('inventoryMonitoringApp').factory('OrderService', ['$q', 'Endpoints', 'Utility', 'moment', factoryFn]);

  function factoryFn($q, endpoints, utility, moment) {
    var service = this;
    var order = {};
    var todayDate = moment().format('YYYY-MM-DD');

    service.fetchOrders = function(accountId, isFutureOrders) {
      var deferred = $q.defer();

      if(order.data && order.isFutureOrders === isFutureOrders) {
        deferred.resolve(_.cloneDeep(order));
      } else {
        endpoints.fetchOrders(accountId, isFutureOrders, todayDate).then(function(response) {
          var orderData;

          order.data = response;
          order.isFutureOrders = isFutureOrders;
          orderData = utility.normalizeArray(order.data, 'Order_vod__c');

          deferred.resolve(orderData);
        }, utility.genericQueryErrorHandler(deferred));
      }

      return deferred.promise;
    };

    service.getAllOrderIds = function() {
      var orderData = utility.normalizeArray(order.data, 'Order_vod__c');

      return orderData.map(function(order){
        return order.Id;
      });
    };

    service.getIsFutureOrderFlag = function() {
      return order.isFutureOrders;
    };

    return service;
  }
})();
