(function() {
  'use strict';

  angular.module('inventoryMonitoringApp').factory('ProductService', ['$q', 'Endpoints', 'Utility', factoryFn]);

  function factoryFn($q, endpoints, utility) {
    var productService = this;
    var products = {};
    var PRODUCT_FIELD_TEXT = 'Product_vod__c';

    productService.fetchProducts = function(productIds) {
      var deferred = $q.defer();

      if(products.data) {
        deferred.resolve(_.cloneDeep(products));
      } else {
          // Query products table to get product name
        endpoints.fetchProducts(productIds).then(function(response) {
          var productList;

          products.data = response;
          productList = utility.normalizeArray(products.data, PRODUCT_FIELD_TEXT);

          deferred.resolve(productList);
        }, utility.genericQueryErrorHandler(deferred));
      }

      return deferred.promise;
    };

    productService.getAllProductIds = function () {
      var productList = utility.normalizeArray(products.data, PRODUCT_FIELD_TEXT);

      return productList.map(function(prod){
        return prod.Id;
      });
    };

    productService.getAllParentIds = function () {
      var productList = utility.normalizeArray(products.data, PRODUCT_FIELD_TEXT);

      var parents = _.groupBy(productList, function(prod) {
        return prod.Parent_Product_vod__c;
      });

      return Object.keys(parents);
    };

    productService.getProductsForFilter = function (selectedBrand) {
      var productList = utility.normalizeArray(products.data, PRODUCT_FIELD_TEXT);

      if (!selectedBrand.Id) {
        return productList;
      }

      return _.filter(productList, function(prod) {
        return prod.Parent_Product_vod__c === selectedBrand.Id;
      });
    };

    productService.getSpecificProduct = function (selectedProduct) {
      var productList = utility.normalizeArray(products.data, PRODUCT_FIELD_TEXT);

      return _.filter(productList, function(prod) {
        return prod.Id === selectedProduct.Id;
      });
    };

    productService.extractIds = function (products) {
      return products.map(function(product){
        return product.Id;
      });
    };

    return productService;
  }
})();
